------------------------------------
-- Shot Glass Raid Frames
------------------------------------
local addonname = "ShotGlass"
local artpath = "Interface\\Addons\\ShotGlass\\Art\\"
local font =					"FONTS\\arialn.ttf"
local rangePollFrequency = .6

-- Clique Support
ClickCastFrames = ClickCastFrames or {}											

-- Function List
local UpdateButton, OnShowButton, OnChangeAttribute, OnEvent, SetHandlers		
local CreateGroupHeader, CreateUnitFrameArtwork
local CallArtworkFunction, SecureInitFunction

local RoleStrings = {
	["DAMAGER"] = DAMAGER,
	["HEALER"] = HEALER,
	["TANK"] = TANK,
}

local Colors = {}

Colors.Grey = {r = .5, g = .5, b = .5,}
Colors.Red = {r = 1, g = 0, b = 0,}
Colors.Orange = {r = 1, g = .5, b = 0,}
Colors.Yellow = {r = 1, g = 1, b = 0,}
Colors.Green = {r = 0, g = 1, b = 0,}

----------------------
-- PollFrameUntil() - Registers a callback, which polls the frame until it expires, and hides the frame.
----------------------

local PollFrameUntil, DelayedStart
do
	local Framelist = {}			-- Key = Frame, Value = Expiration Time
	local Watcherframe = CreateFrame("Frame")
	local WatcherframeActive = false
	local timeToUpdate = 0
	local pollInterval = 1
	
	local function CheckList(self)
		local curTime = GetTime()
		if curTime < timeToUpdate then return end
		local framecount = 0
		timeToUpdate = curTime + pollInterval
		-- Cycle through the watchlist, hiding frames which are timed-out
		for frame, expiration in pairs(Framelist) do
			-- If expired...
			if expiration < curTime then 
				--frame:Hide()
				Framelist[frame] = nil
			-- If active...
			else 
				-- Update the frame
				if frame.Poll then frame:Poll(expiration) end
				framecount = framecount + 1 
			end
		end
		-- If no more frames to watch, unregister the OnUpdate script
		if framecount == 0 then 
			Watcherframe:SetScript("OnUpdate", nil)
			WatcherframeActive = false 
		end
	end
	
	function PollFrameUntil(frame, expiration)
		if expiration == 0 then 
			frame:Hide()
			Framelist[frame] = nil
		else
			Framelist[frame] = expiration
			
			if not WatcherframeActive then 
				Watcherframe:SetScript("OnUpdate", CheckList)
				WatcherframeActive = true
			end
		end
	end
end

------------------------------------
-- Aura Tracking List
------------------------------------
local TrackedAuras = {}

------------------------------------
-- Helper Functions
------------------------------------

-- Abbreviates the Unit Name
local function TruncateName(text) if text then return text:gsub("(%u%l%l%l)(.*)", "%1") end end

-- Shows the mouseover tooltip, out-of-combat
local function ShowUnitTooltip(frame)
	local unitid = frame.UnitID
	if not InCombatLockdown() then
		GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
		GameTooltip:SetUnit(unitid)
		
		role = UnitGroupRolesAssigned(unitid)
		if role and role ~= "NONE" then GameTooltip:AddLine(RoleStrings[role] ) end

		GameTooltip:Show()
	end
end

-- Hides the mouseover tooltip
local function HideUnitTooltip()
	if not InCombatLockdown() then
		GameTooltip:FadeOut()
	end
end

-- Resets all Indicators
local function ResetIndicators(indicators) 
	for name, indicator in pairs(indicators) do 
		if type(indicator)=='table' then indicator:Hide() end
	end
end

-- Polls for Range Check
local function PollOnUpdate(self)
	local unitid = self.UnitID
	if not unitid then return end
	local now = GetTime()
	if now > self.rangeUpdateTime then self.rangeUpdateTime = now + rangePollFrequency else return end
	
	-- Range Check
	if (not IsInGroup()) or UnitInRange(unitid) then self.Graphics:SetScale(1)
	else self.Graphics:SetScale(.45) end
end

-- Hides Party Frames
for i=1,4 do
	local frame = _G["PartyMemberFrame"..i]
	frame:UnregisterAllEvents()
	frame.Show = function() end
	frame:Hide()
end

-- Returns the default assigned color
local ModifiedColor = {}
local function DefaultColorFunction(aura)
	local timeRemaining = aura.expiration - GetTime()
	if timeRemaining < 3 then
		ModifiedColor.r = (aura.color.r*.5)
		ModifiedColor.g = (aura.color.g*.5)
		ModifiedColor.b = (aura.color.b*.5)
		return ModifiedColor
	else
		return aura.color
	end
end

-- Returns Stack Count
local function DefaultTextFunction(aura)
	local text, textcolor

	if stacks and (stacks > 0) then
		text = stacks
		textcolor = Colors.Grey
	else
		text = ""
		textcolor = Colors.Grey
	end

	return text, textcolor
end

-- Applies Color and Stack text to an Indicator
local function ApplyIndicatorStyle(self)
	local auraCache = self.auraCache
	local color, text, textcolor

	-- Color
	if self.colorFunction then color = self.colorFunction(auraCache) else color = DefaultColorFunction(auraCache) end
	if color then self.Image:SetVertexColor(color.r,color.g,color.b,color.a) end
	
	-- Text
	if self.textFunction then 
		text, textcolor = self.textFunction(auraCache) 
	else text, textcolor = DefaultTextFunction(auraCache) end
	self.Text:SetText(text or "") 	
	if textcolor then self.Text:SetTextColor(textcolor.r, textcolor.g, textcolor.b) end
	
	-- Show
	self:Show()
end

local function IndicatorPollFunction(self, expiration)
	if not (self or expiration) then return end
	ApplyIndicatorStyle(self)
end

local function UpdateIndicators(indicators, unitid)
	local auraindex, auraCache, debuffType, indicator, aura
	local priority = {0,0,0,0,0,0,}
	
	ResetIndicators(indicators)

	-- Check Player-Cast Auras
	for auraindex = 1, 40 do
		local name, _, _, stacks, _, _, expires, caster, _, _, spellid = UnitAura(unitid,auraindex,"PLAYER")
		
		if not name then break end
		aura = TrackedAuras[name] 
		if aura then 
			local requestedPriority = aura.priority or 1
			if priority[aura.position] and (requestedPriority > priority[aura.position]) then
				indicator = indicators[aura.position]
				-- Cache data on indicator frame
				auraCache = indicator.auraCache
				auraCache.color, auraCache.stacks, auraCache.expiration = aura.color, stacks, expires
				auraCache.position, auraCache.name, auraCache.unit, auraCache.spellid = aura.position, name, unitid, spellid
				-- Cache delegate functions
				indicator.colorFunction = aura.colorFunction
				indicator.textFunction = aura.textFunction
				-- Update Indicator
				ApplyIndicatorStyle(indicator)				-- Apply the color/scale/textformat
				PollFrameUntil(indicator, expires)			-- Activate timer polling (Not active for general debuff types)
				IndicatorPollFunction(indicator, expires)	-- Update the status immediately
				priority[aura.position] = requestedPriority	
			end
		end
	end
	
	-- Check Debuffs and Debuff Types
	for auraindex = 1, 40 do
		local name, _, _, stacks, debuffType, _, expires, caster, _, _, spellid, _, isBossDebuff =  UnitDebuff(unitid,auraindex)
		if not name then break end
		aura = TrackedAuras[name] or TrackedAuras[debuffType]
		if aura then 
			local requestedPriority = aura.priority or 1
			if priority[aura.position] and (requestedPriority > priority[aura.position]) then
				indicator = indicators[aura.position]
				-- Cache data on indicator frame
				auraCache = indicator.auraCache
				auraCache.color, auraCache.stacks, auraCache.expiration, auraCache.boss = aura.color, stacks, expires, isBossDebuff
				auraCache.position, auraCache.name, auraCache.unit, auraCache.spellid = aura.position, name, unitid, spellid
				-- Update Indicator
				ApplyIndicatorStyle(indicator)
				priority[aura.position] = aura.priority
			end
		end
	end
end

------------------------------------
-- Events
------------------------------------
local UnitEventHandlers = {}

function UnitEventHandlers.UNIT_AURA(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	
	if unit == unitid then 
		UpdateIndicators(self.Graphics.Indicators, unitid)
	end
end

function UnitEventHandlers.UNIT_DISPLAYPOWER(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then 
		local _, powertype = UnitPowerType(unitid or "")
		if powertype == "MANA" then
			self.Graphics.Power:Show()
			self.Graphics.Power:SetMinMaxValues(0,UnitPowerMax(unitid))
			self.Graphics.Power:SetValue(UnitPower(unitid))
		else
			self.Graphics.Power:Hide()
		end
	end
end

function UnitEventHandlers.UNIT_POWER(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then 
		self.Graphics.Power:SetValue(UnitPower(unitid))
	end
end

function UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then
		if UnitIsDead(unitid) or UnitIsGhost(unitid) or (not UnitIsConnected(unitid))then 
			self.Graphics.Health:SetAlpha(.7)
			self.Graphics.Power:Hide()
		else 
			self.Graphics.Health:SetAlpha(1)
		end
	end
end
UnitEventHandlers.PARTY_MEMBER_ENABLE = UnitEventHandlers.PARTY_MEMBER_DISABLE
UnitEventHandlers.PLAYER_DEAD = UnitEventHandlers.PARTY_MEMBER_DISABLE

function UnitEventHandlers.UNIT_HEALTH(self, eventunit)
	local unitid = self.UnitID
	if not unitid then return end
	local petunit, visibleunit
	if unitid == "player" then petunit = "pet" else petunit = unitid.."pet" end

	if eventunit == unitid or eventunit == petunit then 
		if UnitHasVehicleUI(unitid) then visibleunit = petunit
		else visibleunit = unitid end
		if UnitExists(visibleunit) then
			-- Sets Health Bar
			self.Graphics.Health:SetMinMaxValues(0,UnitHealthMax(visibleunit))
			self.Graphics.Health:SetValue(UnitHealth(visibleunit))	
		end
	end
end
UnitEventHandlers.UNIT_MAXHEALTH = UnitEventHandlers.UNIT_HEALTH

function UnitEventHandlers.PLAYER_TARGET_CHANGED(self) 
	local unitid = self.UnitID
	if not unitid then return end
	local visibleunit
	
	if UnitHasVehicleUI(unitid) then
		if unitid == "player" then visibleunit = "pet" else visibleunit = unitid.."pet" end
	else visibleunit = unitid end
	
	-- Sets Target Highlight
	if UnitIsUnit("target", visibleunit) then self.Graphics.Border:Show()
	else self.Graphics.Border:Hide() end		
end

function UnitEventHandlers.UNIT_THREAT_SITUATION_UPDATE(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	local unitaggro = UnitThreatSituation(unitid)
	if unitaggro and unitaggro > 1 then 
		self.Graphics.Aggro:Show() 
	else 
		self.Graphics.Aggro:Hide() 
	end
end
	
--[[	Search for RAID_TARGET_UPDATE
	
	-- Sets the raid icon texture coordinates
local function SetRaidTargetIconCoord (texture, index)
	local left, right, top, bottom;
	left = mod((index-1) , 4) * .25;
	right = left + 0.25;
	top = floor((index-1) / 4) * .5;
	bottom = top + 0.5;
	texture:SetTexCoord(left, right, top, bottom);
end

function UnitEventHandlers.RAID_TARGET_UPDATE(self)
	local unitid = self.UnitID
	if not unitid then return end

	local index = GetRaidTargetIndex(unitid)
	if index then
		SetRaidTargetIconCoord(self.Graphics.RaidIcon, index)
		self.Graphics.RaidIcon:Show()
	else self.Graphics.RaidIcon:Hide() end
end
--]]

function UnitEventHandlers.UNIT_PORTRAIT_UPDATE(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	UnitEventHandlers.UNIT_DISPLAYPOWER(self, unitid)
	UnitEventHandlers.UNIT_AURA(self, unitid)
	UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)	
	UnitEventHandlers.UNIT_HEALTH(self, unitid)
	-- UnitEventHandlers.RAID_TARGET_UPDATE(self)
end
UnitEventHandlers.PLAYER_UNGHOST = UnitEventHandlers.UNIT_PORTRAIT_UPDATE
UnitEventHandlers.PLAYER_ALIVE = UnitEventHandlers.UNIT_PORTRAIT_UPDATE

function UnitEventHandlers.UNIT_FLAGS(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	UnitEventHandlers.UNIT_AURA(self, unitid)
	UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)
	
end

function UnitEventHandlers.UNIT_ENTERED_VEHICLE(self, unit)
	OnChangeAttribute(self, "unit")
	if self.UnitID then UpdateButton(self, self.UnitID) end
end
UnitEventHandlers.UNIT_EXITED_VEHICLE = UnitEventHandlers.UNIT_ENTERED_VEHICLE
UnitEventHandlers.UNIT_PET = UnitEventHandlers.UNIT_ENTERED_VEHICLE
function UnitEventHandlers.UNIT_HEAL_PREDICTION(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	
	if unit == unitid then
		local incomingHeal = UnitGetIncomingHeals(unitid) or 0
		if incomingHeal > 0 then incomingHeal = UnitHealth(unitid)+incomingHeal end
		self.Graphics.IncomingHeal:SetValue(incomingHeal)	
	end
end

------------------------------------
-- Update Functions
------------------------------------
function UpdateButton(self, unitid)
		local class, color, Graphics
		self.Nextupdate = 0
		Graphics = self.Graphics
		-- Colors
		class = select(2, UnitClass(unitid)) or "WARRIOR"
		color = RAID_CLASS_COLORS[class]
		-- Name
		Graphics.Text:SetText(TruncateName(UnitName(unitid)))
		-- Health
		local maxhealth = UnitHealthMax(unitid)
		Graphics.Health:SetStatusBarColor(0.05, 0.05, 0.05, 1)
		Graphics.Health:SetMinMaxValues(0,maxhealth)
		-- Incoming Heals
		Graphics.IncomingHeal:SetStatusBarColor(0.05, 0.05, 0.05, .6)
		Graphics.IncomingHeal:SetMinMaxValues(0,maxhealth)
		-- Power		
		UnitEventHandlers.UNIT_DISPLAYPOWER(self, unitid)
		UnitEventHandlers.UNIT_AURA(self, unitid)
		UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)	
		UnitEventHandlers.UNIT_HEALTH(self, unitid)	
		UnitEventHandlers.UNIT_HEAL_PREDICTION(self, unitid)	
		--UnitEventHandlers.RAID_TARGET_UPDATE(self)
		
		-- Vehicle
		if UnitHasVehicleUI(unitid) then		-- Vehicle Support
			Graphics.Health:SetBackdropColor(0, 1, 0, 1)
			Graphics.Text:SetTextColor(0, .5, 0, 1)
		else
			Graphics.Text:SetTextColor(color.r*.5, color.g*.5, color.b*.5, 1)
			Graphics.Health:SetBackdropColor(color.r, color.g, color.b, 1)
		end
end

function SetHandlers(self)
	self:SetScript("OnShow", OnShowButton)
	self:SetScript("OnEvent", OnEvent)
	self:SetScript("OnUpdate", PollOnUpdate)
	for eventname, _ in pairs(UnitEventHandlers) do self:RegisterEvent(eventname) end
end

function OnShowButton(self)
	if not InCombatLockdown() then self:SetAttribute('*type1','target') end
	
	local unitid = self:GetAttribute("unit")
	if unitid then 
		self.UnitID = unitid
		UpdateButton(self, unitid)
	end
end

function OnEvent(self, event, ...)
	UnitEventHandlers[event](self, ...) 
end

function OnChangeAttribute(self, attribute, value)
	if attribute == "_wrapentered" then return	-- Clique filter
	elseif attribute == "unit" then 
		if not InCombatLockdown() then 
			self:SetAttribute('*type1','target')
			self:SetAttribute("toggleForVehicle", true)		-- Vehicle Support
		end
		
		ClickCastFrames[self] = true		-- Clique
		SetHandlers(self)
		
		local unitid = self:GetAttribute("unit")
		if unitid then 
			self.UnitID = unitid
			UpdateButton(self, unitid) 
		end
	end
end

------------------------------------
-- Unit Frame Artwork
------------------------------------
function CreateUnitFrameArtwork(self)
	local Graphics
	self:EnableMouse(true)
	self.info = {}
	
	-- Create Base Frames
	self.AuraWatcher = CreateFrame("Frame", nil, self)
	self.Graphics = CreateFrame("Frame", nil, self)
	self.auraUpdateTime = 0
	self.rangeUpdateTime = 0
	
	Graphics = self.Graphics
	Graphics:SetPoint("CENTER", self)
	Graphics:SetWidth(64)
	Graphics:SetHeight(64)
		
	-- Health Bar 
	Graphics.Health = CreateFrame("StatusBar", nil, Graphics)
	Graphics.Health:SetStatusBarTexture(artpath.."HealthStatusbar")
	Graphics.Health:SetBackdrop({bgFile = artpath.."HealthStatusbar"});
	Graphics.Health:SetOrientation("VERTICAL")
	Graphics.Health:SetPoint("CENTER", 0, -1)
	Graphics.Health:SetFrameLevel(Graphics:GetFrameLevel()-1)
	Graphics.Health:SetWidth(36.5)
	Graphics.Health:SetHeight(34)
	
	-- Heal Prediction Bar 
	Graphics.IncomingHeal = CreateFrame("StatusBar", nil, Graphics.Health)
	Graphics.IncomingHeal:SetStatusBarTexture(artpath.."HealthStatusbar")
	Graphics.IncomingHeal:SetOrientation("VERTICAL")
	Graphics.IncomingHeal:SetAllPoints(Graphics.Health)
	
	-- Overlay Anchor (To make layering easier)
	Graphics.OverlayAnchor = CreateFrame("StatusBar", nil, Graphics.Health)
	Graphics.OverlayAnchor:SetAllPoints()
	Graphics.OverlayAnchor:SetFrameLevel(Graphics.IncomingHeal:GetFrameLevel()+1)
	
	-- Power (Mana) Bar
	Graphics.Power = CreateFrame("StatusBar", nil, Graphics.OverlayAnchor)
	Graphics.Power:SetStatusBarTexture(artpath.."ManaStatusbar")
	Graphics.Power:SetBackdrop({bgFile = artpath.."ManaStatusbar"});
	Graphics.Power:SetOrientation("VERTICAL")
	Graphics.Power:SetPoint("CENTER", 20, 7)
	Graphics.Power:SetHeight(30)	
	Graphics.Power:SetWidth(16)		
	Graphics.Power:SetBackdropColor(0, 0, 0, 1)
	Graphics.Power:SetStatusBarColor(4/255, 78/255, 141/255, 1)

	Graphics.Power.Border = Graphics.Power:CreateTexture(nil, "OVERLAY")
	Graphics.Power.Border:SetTexture(artpath.."ManaOverlay")
	--Graphics.Power.Border:SetPoint("TOPLEFT", -1,1)	
	Graphics.Power.Border:SetPoint("TOPLEFT", 0,0)	
	Graphics.Power.Border:SetHeight(32)	
	Graphics.Power.Border:SetWidth(16)	

	-- Create Name Text Field
	Graphics.Text = Graphics.OverlayAnchor:CreateFontString(nil, "OVERLAY")
	Graphics.Text:SetFont(font, 12)
	Graphics.Text:SetPoint("BOTTOM", Graphics.Health, "BOTTOM", -1, 1)
	Graphics.Text:SetWidth(50)
	Graphics.Text:SetHeight(13)
	Graphics.Text:SetShadowColor(0,0,0,1) 
	Graphics.Text:SetShadowOffset(1,-1) 

	-- Overlay Artwork
	Graphics.Overlay = Graphics.OverlayAnchor:CreateTexture(nil, "ARTWORK")
	Graphics.Overlay:SetTexture(artpath.."HealthOverlay")
	Graphics.Overlay:SetAllPoints(Graphics)
	
	-- [[ Raid Icon
	Graphics.RaidIcon = Graphics.OverlayAnchor:CreateTexture(nil, "OVERLAY")
	Graphics.RaidIcon:SetTexture(artpath.."RaidIcons")
	--Graphics.RaidIcon:SetPoint("TOP", Graphics.Health, "TOP", 0, -5)
	Graphics.RaidIcon:SetPoint("CENTER", Graphics.Health, "LEFT", -1, 3)
	Graphics.RaidIcon:SetSize(14, 14)
	--Graphics.RaidIcon:SetAlpha(.6)
	Graphics.RaidIcon:Hide()
	--]]
	
	-- Border/Selection Box Artwork
	Graphics.Border = Graphics:CreateTexture(nil, "ARTWORK")
	Graphics.Border:SetTexture(artpath.."Select")
	Graphics.Border:SetAllPoints()
	Graphics.Border:Hide()

	Graphics.Indicators = {}
	local Indicators = Graphics.Indicators
	
	-- Indicators Table
	Graphics.IndicatorLayer = CreateFrame("Frame", nil, Graphics)
	Graphics.IndicatorLayer:SetAllPoints(Graphics)
	Graphics.IndicatorLayer:Show()
	
	local Indicator, IndicatorIndex
	for index = 1, 6 do
		Indicators[index] = CreateFrame("Frame", nil, Graphics.IndicatorLayer)
		Indicator = Indicators[index]
		Indicator:SetWidth(16)
		Indicator:SetHeight(16)
		Indicator.Image = Indicator:CreateTexture(nil, "ARTWORK")
		Indicator.Image:SetAllPoints()
		Indicator.Text = Indicator:CreateFontString(nil, "OVERLAY")
		Indicator.Text:SetFont(font, 9, "OUTLINE")
		Indicator.Text:SetPoint("TOP", Indicator, "BOTTOM", 1, 5.5)
		Indicator.Text:SetJustifyH("CENTER")
		Indicator.Text:SetWidth(12)
		Indicator.Text:SetHeight(12)
		Indicator.Poll = IndicatorPollFunction
		Indicator.auraCache = {}
		Indicator:Hide()	
	end
	
	for index = 1, 3 do Indicators[index].Image:SetTexture(artpath.."BuffDot") end
	for index = 2, 3 do Indicators[index]:SetPoint("RIGHT", Indicators[index-1], "CENTER", -1, 0) end
	Indicators[1]:SetPoint("TOP", 6, -7) -- +16)
	
	for index = 4, 6 do Indicators[index].Image:SetTexture(artpath.."DebuffArrow") end
	for index = 5, 6 do Indicators[index]:SetPoint("RIGHT", Indicators[index-1], "CENTER", -3, 0) end
	Indicators[4]:SetPoint("BOTTOM", 11, 4) -- +16)
	
	-- Aggro/Outline Artwork
	Graphics.Aggro = Graphics:CreateTexture(nil, "OVERLAY")
	Graphics.Aggro:SetTexture(artpath.."Aggro")
	Graphics.Aggro:SetVertexColor(1,0,0,1)
	Graphics.Aggro:SetAllPoints()
	Graphics.Aggro:Hide()
	
	-- Scripts
	self:SetScript("OnAttributeChanged", OnChangeAttribute)
end

-- Calls the Unit Frame Artwork function; Called by the secure config function
function CallArtworkFunction(self, frameName, ...)
	local frame = _G[frameName]
	CreateUnitFrameArtwork(frame)
	
	frame:SetScript("OnEnter", ShowUnitTooltip )
	frame:SetScript("OnLeave", HideUnitTooltip )
end

-- Initialization Function: Set Size and other properties of the Unit Button
-- Variables from addons are not accessible when this function is called.
SecureInitFunction = [[
	local header = self:GetParent()
	header:CallMethod('CallArtworkFunction', self:GetName())
	self:SetHeight(45)
	self:SetWidth(45)
]]

-- Creates the Group Header, which dynamically controls the creation of Unit Frames
function CreateGroupHeader()
	local RaidGroupHeader = CreateFrame("Frame", addonname.."_GroupHeader", UIParent, "SecureGroupHeaderTemplate")
	local HandleFrame = CreateFrame("Button", addonname.."_Handle", RaidGroupHeader)
	
	RaidGroupHeader:SetPoint("CENTER", UIParent)
	RaidGroupHeader:SetMovable(true)
	RaidGroupHeader:SetClampedToScreen(true)

	-- RaidGroupHeader Frame Header
	RaidGroupHeader:SetAttribute("template", "SecureUnitButtonTemplate")
	RaidGroupHeader:SetAttribute("templateType", "Button")
	RaidGroupHeader:SetAttribute("groupFilter", "1,2,3,4,5,6,7,8")
	RaidGroupHeader:SetAttribute("showRaid", true) -- When true, the group header is shown when the player is in a raid.
	RaidGroupHeader:SetAttribute("showParty", true) -- When true, the group header is shown when the player is in a party. This attribute doesnt imply showRaid but can work alongside it.
	RaidGroupHeader:SetAttribute("showSolo", true) -- When true, the header is shown when the player is not in any group. This option implies showPlayer.
	RaidGroupHeader:SetAttribute("showPlayer", true) -- When true, the header includes the player when not in a raid (normally, the player would not be visible in a party listing).
	RaidGroupHeader:SetAttribute("point", "RIGHT")	-- The anchor point per row
	RaidGroupHeader:SetAttribute("unitsPerColumn",5)
	RaidGroupHeader:SetAttribute("maxColumns",8)
	RaidGroupHeader:SetAttribute("columnAnchorPoint","TOP") -- The anchor point for each new column. A value of LEFT will cause the columns to grow to the right.
	RaidGroupHeader.CallArtworkFunction = CallArtworkFunction	-- This function is called from the init function
	RaidGroupHeader:SetAttribute("initialConfigFunction", SecureInitFunction) 		-- Init function (Secure)
			
	-- Handle Frame
	local defaultAlpha, partialAlpha = .55, .75
	HandleFrame:SetPoint("TOP", RaidGroupHeader, "TOP", 0, -4)
	HandleFrame:SetPoint("RIGHT", RaidGroupHeader, "RIGHT", 18, 0)
	HandleFrame:SetWidth(16)
	HandleFrame:SetHeight(16)
	HandleFrame:SetAlpha(defaultAlpha)

	HandleFrame.Texture = HandleFrame:CreateTexture(nil, "ARTWORK")
	HandleFrame.Texture:SetTexture(artpath.."CornerHandle")
	HandleFrame.Texture:SetAllPoints(HandleFrame)
	
	HandleFrame:SetClampedToScreen(true)
	HandleFrame:RegisterForClicks("LeftButtonUp","RightButtonUp")
	HandleFrame:EnableMouse(true)	-- /run ShotGlassRaidFrames.HandleFrame:EnableMouse(false) -- to lock the frame, in game
	HandleFrame:SetScript("OnClick",function(self, button) if button == "RightButton" then InterfaceOptionsFrame_OpenToCategory("Shot Glass") end end)
	HandleFrame:SetScript("OnMouseDown",function(self, button) HandleFrame:SetAlpha(1); if button =="LeftButton" then RaidGroupHeader:StartMoving() end end)
	HandleFrame:SetScript("OnMouseUp",function(self, button) HandleFrame:SetAlpha(partialAlpha); RaidGroupHeader:StopMovingOrSizing() end)
	
	HandleFrame:SetScript("OnEnter",function(self) 	
		HandleFrame:SetAlpha(.8)
		GameTooltip_SetDefaultAnchor(GameTooltip,UIParent)
		GameTooltip:AddLine("Shot Glass")
		GameTooltip:AddLine("Click and Drag to Move|nRight-Click to open Config Panel",1,1,1,1)
		--GameTooltip:Show()
	end)
	HandleFrame:SetScript("OnLeave",function(self) 
		HandleFrame:SetAlpha(defaultAlpha)
		--GameTooltip:Hide() 
	end)
	
	RaidGroupHeader.HandleFrame = HandleFrame
	
	RaidGroupHeader:Show()
	return RaidGroupHeader
end

ShotGlassRaidFrames = CreateGroupHeader()
ShotGlassRaidFrames.TrackedAuras = TrackedAuras

